//  ---------------------------------------------------------------------------
//
//  @file       TwEventGLFW.c
//  @brief      Helper: 
//              translate and re-send mouse and keyboard events 
//              from GLFW event callbacks to AntTweakBar
//  
//  @author     Philippe Decaudin
//  @license    This file is part of the AntTweakBar library.
//              For conditions of distribution and use, see License.txt
//
//  ---------------------------------------------------------------------------

#include <GLFW/glfw3.h>

#include <LeanTweakBar.h>


static int translate(int key)
{

    switch (key) {
        case GLFW_KEY_SPACE:
            return TW_KEY_SPACE;

        case GLFW_KEY_ESCAPE:
            return TW_KEY_ESCAPE;
        case GLFW_KEY_ENTER:
            return TW_KEY_RETURN;
        case GLFW_KEY_TAB:
            return TW_KEY_TAB;
        case GLFW_KEY_BACKSPACE:
            return TW_KEY_BACKSPACE;
        case GLFW_KEY_INSERT:
            return TW_KEY_INSERT;
        case GLFW_KEY_DELETE:
            return TW_KEY_DELETE;
        case GLFW_KEY_RIGHT:
            return TW_KEY_RIGHT;
        case GLFW_KEY_LEFT:
            return TW_KEY_LEFT;
        case GLFW_KEY_DOWN:
            return TW_KEY_DOWN;
        case GLFW_KEY_UP:
            return GLFW_KEY_UP;
        case GLFW_KEY_PAGE_UP:
            return TW_KEY_PAGE_UP;
        case GLFW_KEY_PAGE_DOWN:
            return TW_KEY_PAGE_DOWN;
        case GLFW_KEY_HOME:
            return TW_KEY_HOME;
        case GLFW_KEY_END:
            return TW_KEY_END;
        case GLFW_KEY_PAUSE:
            return TW_KEY_PAUSE;

        case GLFW_KEY_F1:
        case GLFW_KEY_F2:
        case GLFW_KEY_F3:
        case GLFW_KEY_F4:
        case GLFW_KEY_F5:
        case GLFW_KEY_F6:
        case GLFW_KEY_F7:
        case GLFW_KEY_F8:
        case GLFW_KEY_F9:
        case GLFW_KEY_F10:
        case GLFW_KEY_F11:
        case GLFW_KEY_F12:
        case GLFW_KEY_F13:
        case GLFW_KEY_F14:
        case GLFW_KEY_F15:
            return TW_KEY_F1 + (key - GLFW_KEY_F1);

        default:
            return key;
    }
}



int TwEventKeyGLFW3(GLFWwindow* window, int key, int scancode, int action, int mods)
{
    int twMod = 0;

    if (mods & GLFW_MOD_SHIFT)
        twMod |= TW_KMOD_SHIFT;
    else
        twMod &= ~TW_KMOD_SHIFT;

    if (mods & GLFW_MOD_CONTROL)
        twMod |= TW_KMOD_CTRL;
    else
        twMod &= ~TW_KMOD_CTRL;

    if (mods & GLFW_MOD_ALT)
        twMod |= TW_KMOD_ALT;
    else
        twMod &= ~TW_KMOD_ALT;

    // Process key
    if (action == GLFW_PRESS)
        return TwKeyPressed(translate(key), twMod);
    return 0;
}


int TwEventCharGLFW3(GLFWwindow* window, unsigned int codepoint)
{
    return TwKeyPressed(codepoint, 0);
}


int TwEventMousePosGLFW3(GLFWwindow* window, double xpos, double ypos)
{
    return TwMouseMotion(xpos, ypos);
}


int TwEventMouseButtonGLFW3(GLFWwindow* window, int button, int action, int mods)
{
    TwMouseAction act = (action==GLFW_PRESS) ? TW_MOUSE_PRESSED : TW_MOUSE_RELEASED;

    if( button==GLFW_MOUSE_BUTTON_LEFT )
        return TwMouseButton(act, TW_MOUSE_LEFT);
    else if( button==GLFW_MOUSE_BUTTON_RIGHT )
        return TwMouseButton(act, TW_MOUSE_RIGHT);
    else if( button==GLFW_MOUSE_BUTTON_MIDDLE )
        return TwMouseButton(act, TW_MOUSE_MIDDLE);
    return 0;
}


int TwEventMouseWheelGLFW3(GLFWwindow* window, double xoffset, double yoffset)
{
    return TwMouseWheel(yoffset);
}


static void wrapTwEventKeyGLFW3(GLFWwindow* window, int key, int scancode, int action, int mods)
{
    TwEventKeyGLFW3(window, key, scancode, action, mods);
}

static void wrapTwEventCharGLFW3(GLFWwindow* window, unsigned int codepoint)
{
    TwEventCharGLFW3(window, codepoint);
}

static void wrapTwEventMousePosGLFW3(GLFWwindow* window, double xpos, double ypos)
{
    TwEventMousePosGLFW3(window, xpos, ypos);
}

static void wrapTwEventMouseButtonGLFW3(GLFWwindow* window, int button, int action, int mods)
{
    TwEventMouseButtonGLFW3(window, button, action, mods);
}

static void wrapTwEventMouseWheelGLFW3(GLFWwindow* window, double xoffset, double yoffset)
{
    TwEventMouseWheelGLFW3(window, xoffset, yoffset);
}

void TwInitGLFW3(GLFWwindow* window)
{
    glfwSetKeyCallback(window, wrapTwEventKeyGLFW3);
    glfwSetCharCallback(window, wrapTwEventCharGLFW3);
    glfwSetCursorPosCallback(window, wrapTwEventMousePosGLFW3);
    glfwSetMouseButtonCallback(window, wrapTwEventMouseButtonGLFW3);
    glfwSetScrollCallback(window, wrapTwEventMouseWheelGLFW3);
}
